==> _private_chat-1.0.7.tar.gz: lib/chat_detail/chat_helper.dart <==
import 'package:cloud_firestore/cloud_firestore.dart';
import 'package:encrypt/encrypt.dart';
import 'package:get/get.dart';
import 'package:intl/intl.dart';
import 'package:_private_core/_private_core.dart';
import 'package:uuid/uuid.dart';
import 'package:_private_chat/constant.dart';

import '../utils/utils.dart';
import 'chat_detail_screen.dart';

class ChatHelper {
  ChatHelper._();

  //Manage status message sending
  static final Map<String, Future> _messageStatus = <String, Future>{};

  static _getMessageKey(String? peerNo, int? timestamp) => '$peerNo$timestamp';

  static getMessageStatus(String? peerNo, int? timestamp) {
    final key = _getMessageKey(peerNo, timestamp);
    return _messageStatus[key] ?? true;
  }

  static addMessage(String? peerNo, int? timestamp, Future future) {
    final key = _getMessageKey(peerNo, timestamp);
    future.then((_) {
      _messageStatus.remove(key);
    });
    _messageStatus[key] = future;
  }

  //===========
  static Uuid uuid = const Uuid();
  static String generateConversationId() => uuid.v4();

  static final _iv = IV.fromLength(16);
  static String generateEncryptKey() => Key.fromSecureRandom(32).base64;

  static String encrypt(key, input) {
    try {
      return Encrypter(AES(Key.fromBase64(key))).encrypt(input, iv: _iv).base64;
    } catch (e) {
      return input;
    }
  }

  static String decrypt(key, input) {
    try {
      return Encrypter(AES(Key.fromBase64(key)))
          .decrypt(Encrypted.from64(input), iv: _iv);
    } catch (e) {
      return input;
    }
  }

  static findConversationById(
      {required String conversationId,
      required ChatConversationType type}) async {
    QuerySnapshot q = await FirebaseFirestore.instance
        .collection(kCollectionCONVERSATIONS)
        .where(kDbCONVERSATIONTYPE,
            isEqualTo: type == ChatConversationType.group ? kDbGroup : kDbSOLO)
        .where(kDbCONVERSATIONID, isEqualTo: conversationId)
        .get();
    if (q.size > 0) {
      return q.docs.first.data();
    }
  }

  static findOrCreateSoloConversation(username, {firebaseId}) async {
    QuerySnapshot q = await FirebaseFirestore.instance
        .collection(kCollectionCONVERSATIONS)
        .where(kDbCONVERSATIONTYPE, isEqualTo: kDbSOLO)
        .where(kDbMEMBERS, arrayContains: loggedUsername)
        // .where(kDbMEMBERS, arrayContains: peerId)
        .get();
    if (q.docs.any(
        (d) => ((d.data() as Map)[kDbMEMBERS] as List).contains(username))) {
      return q.docs
          .firstWhere(
              (d) => ((d.data() as Map)[kDbMEMBERS] as List).contains(username))
          .data();
    } else {
      String id = generateConversationId();
      Map<String, dynamic> data = {
        kDbCONVERSATIONID: id,
        kDbCONVERSATIONTYPE: kDbSOLO,
        kDbLASTMESSAGETIME: 0,
        kDbENCRYPTKEY: generateEncryptKey(),
        kDbMEMBERS: [username, loggedUsername],
        kDbMEMBERSFIREBASEID: [firebaseId, loggedFirebaseId],
        kDbMEMBERSDETAIL: [
          {kDbUSERNAME: username},
          {kDbUSERNAME: loggedUsername}
        ],
      };
      await FirebaseFirestore.instance
          .collection(kCollectionCONVERSATIONS)
          .doc(id)
          .set(data);
      return data;
    }
  }

  static findOrCreateGroupConversation({
    String? conversationId,
    peers,
    peersFirebaseId,
    peersDetail,
    groupName,
    thumbnail,
  }) async {
    QuerySnapshot q = await FirebaseFirestore.instance
        .collection(kCollectionCONVERSATIONS)
        .where(kDbCONVERSATIONTYPE, isEqualTo: kDbGroup)
        .where(kDbCONVERSATIONID, isEqualTo: conversationId ?? '-1')
        .get();
    if (q.size > 0) {
      return q.docs.first.data();
    } else {
      String id = conversationId ?? generateConversationId();
      Map<String, dynamic> data = {
        kDbCONVERSATIONID: id,
        kDbCONVERSATIONTYPE: kDbGroup,
        kDbGROUPNAME: groupName,
        kDbTHUMBNAIL: thumbnail,
        kDbLASTMESSAGETIME: 0,
        kDbENCRYPTKEY: generateEncryptKey(),
        kDbMEMBERS: [...peers, loggedUsername],
        kDbMEMBERSFIREBASEID: [...peersFirebaseId, loggedFirebaseId],
        kDbMEMBERSDETAIL: [
          ...peersDetail,
          {kDbUSERNAME: loggedUsername}
        ],
      };
      await FirebaseFirestore.instance
          .collection(kCollectionCONVERSATIONS)
          .doc(id)
          .set(data);
      return data;
    }
  }

  static updateConversationInfo(conversationId, data) {
    FirebaseFirestore.instance
        .collection(kCollectionCONVERSATIONS)
        .doc(conversationId)
        .update(data);
  }

  static setLastMessageTime(conversationData, lastTime,
      {alsoMembersDetail = false}) {
    Map<String, dynamic> data = {kDbLASTMESSAGETIME: lastTime};
    if (alsoMembersDetail) {
      List members = conversationData[kDbMEMBERSDETAIL];
      members.firstWhere(
          (e) => e[kDbUSERNAME] == loggedUsername)[kDbTIMESTAMP] = lastTime + 1;
      data.addAll({kDbMEMBERSDETAIL: members});
    }
    FirebaseFirestore.instance
        .collection(kCollectionCONVERSATIONS)
        .doc(conversationData[kDbCONVERSATIONID])
        .update(data);
  }

  static setLastTimeTimestamp(conversationData, [timestamp]) async {
    if (conversationData == null) return;
    List members = conversationData[kDbMEMBERSDETAIL];
    members.firstWhere((e) => e[kDbUSERNAME] == loggedUsername)[kDbTIMESTAMP] =
        timestamp ??
            DateTime.now()
                .add(const Duration(seconds: 1))
                .millisecondsSinceEpoch;
    await FirebaseFirestore.instance
        .collection(kCollectionCONVERSATIONS)
        .doc(conversationData[kDbCONVERSATIONID])
        .update({kDbMEMBERSDETAIL: members});
  }

  static removeConversation(id) async {
    FirebaseFirestore.instance
        .collection(kCollectionCONVERSATIONS)
        .doc(id)
        .delete();
  }

  static getWhen(DateTime date) {
    DateTime now = DateTime.now();
    String when;
    if (date.isToday()) {
      when = 'today'.tr;
    } else if (date.day == now.subtract(const Duration(days: 1)).day) {
      when = 'yesterday'.tr;
    } else {
      when = DateFormat.MMMd().format(date);
    }
    return when;
  }
}

==> _private_chat-1.0.8.tar.gz: lib/chat_detail/chat_helper.dart <==
import 'package:cloud_firestore/cloud_firestore.dart';
import 'package:encrypt/encrypt.dart';
import 'package:get/get.dart';
import 'package:intl/intl.dart';
import 'package:_private_core/_private_core.dart';
import 'package:uuid/uuid.dart';
import 'package:_private_chat/constant.dart';

import '../utils/utils.dart';
import 'chat_detail_screen.dart';

CollectionReference get colConversation =>
    FirebaseFirestore.instance.collection(kCollectionCONVERSATIONS);

class ChatHelper {
  ChatHelper._();

  //Manage status message sending
  static final Map<String, Future> _messageStatus = <String, Future>{};

  static _getMessageKey(String? peerNo, int? timestamp) => '$peerNo$timestamp';

  static getMessageStatus(String? peerNo, int? timestamp) {
    final key = _getMessageKey(peerNo, timestamp);
    return _messageStatus[key] ?? true;
  }

  static addMessage(String? peerNo, int? timestamp, Future future) {
    final key = _getMessageKey(peerNo, timestamp);
    future.then((_) {
      _messageStatus.remove(key);
    });
    _messageStatus[key] = future;
  }

  //===========
  static Uuid uuid = const Uuid();
  static String generateConversationId() => uuid.v4();

  static final _iv = IV.fromLength(16);
  static String generateEncryptKey() => Key.fromSecureRandom(32).base64;

  static String encrypt(key, input) {
    try {
      return Encrypter(AES(Key.fromBase64(key))).encrypt(input, iv: _iv).base64;
    } catch (e) {
      return input;
    }
  }

  static String decrypt(key, input) {
    try {
      return Encrypter(AES(Key.fromBase64(key)))
          .decrypt(Encrypted.from64(input), iv: _iv);
    } catch (e) {
      return input;
    }
  }

  static findConversationById(
      {required String conversationId,
      required ChatConversationType type}) async {
    QuerySnapshot q = await colConversation
        .where(kDbCONVERSATIONTYPE,
            isEqualTo: type == ChatConversationType.group ? kDbGroup : kDbSOLO)
        .where(kDbCONVERSATIONID, isEqualTo: conversationId)
        .get();
    if (q.size > 0) {
      return q.docs.first.data();
    }
  }

  static findOrCreateSoloConversation(username, {firebaseId}) async {
    QuerySnapshot q = await colConversation
        .where(kDbCONVERSATIONTYPE, isEqualTo: kDbSOLO)
        .where(kDbMEMBERS, arrayContains: loggedUsername)
        // .where(kDbMEMBERS, arrayContains: peerId)
        .get();
    if (q.docs.any(
        (d) => ((d.data() as Map)[kDbMEMBERS] as List).contains(username))) {
      return q.docs
          .firstWhere(
              (d) => ((d.data() as Map)[kDbMEMBERS] as List).contains(username))
          .data();
    } else {
      String id = generateConversationId();
      Map<String, dynamic> data = {
        kDbCONVERSATIONID: id,
        kDbCONVERSATIONTYPE: kDbSOLO,
        kDbLASTMESSAGETIME: 0,
        kDbENCRYPTKEY: generateEncryptKey(),
        kDbMEMBERS: [username, loggedUsername],
        kDbMEMBERSFIREBASEID: [firebaseId, loggedFirebaseId],
        kDbMEMBERSDETAIL: [
          {kDbUSERNAME: username},
          {kDbUSERNAME: loggedUsername}
        ],
      };
      await colConversation.doc(id).set(data);
      return data;
    }
  }

  static findOrCreateGroupConversation({
    String? conversationId,
    peers,
    peersFirebaseId,
    peersDetail,
    groupName,
    thumbnail,
  }) async {
    QuerySnapshot q = await colConversation
        .where(kDbCONVERSATIONTYPE, isEqualTo: kDbGroup)
        .where(kDbCONVERSATIONID, isEqualTo: conversationId ?? '-1')
        .get();
    if (q.size > 0) {
      return q.docs.first.data();
    } else {
      String id = conversationId ?? generateConversationId();
      Map<String, dynamic> data = {
        kDbCONVERSATIONID: id,
        kDbCONVERSATIONTYPE: kDbGroup,
        kDbGROUPNAME: groupName,
        kDbTHUMBNAIL: thumbnail,
        kDbLASTMESSAGETIME: 0,
        kDbENCRYPTKEY: generateEncryptKey(),
        kDbMEMBERS: [...peers, loggedUsername],
        kDbMEMBERSFIREBASEID: [...peersFirebaseId, loggedFirebaseId],
        kDbMEMBERSDETAIL: [
          ...peersDetail,
          {kDbUSERNAME: loggedUsername}
        ],
      };
      await colConversation.doc(id).set(data);
      return data;
    }
  }

  static updateConversationInfo(conversationId, data) {
    colConversation.doc(conversationId).update(data);
  }

  static setFavorite(
      {required conversationId, required bool status, conversationData}) async {
    var doc =
        conversationData ?? await colConversation.doc(conversationId).get();
    List users = doc[kDbFavorite] ?? [];
    if (status) {
      if (users.contains(loggedUsername)) return;
      users.add(loggedUsername);
    } else {
      if (!users.contains(loggedUsername)) return;
      users.remove(loggedUsername);
    }
    colConversation.doc(conversationId).update({kDbFavorite: users});
  }

  static setLastMessageTime(conversationData, lastTime,
      {alsoMembersDetail = false}) {
    Map<String, dynamic> data = {kDbLASTMESSAGETIME: lastTime};
    if (alsoMembersDetail) {
      List members = conversationData[kDbMEMBERSDETAIL];
      members.firstWhere(
          (e) => e[kDbUSERNAME] == loggedUsername)[kDbTIMESTAMP] = lastTime + 1;
      data.addAll({kDbMEMBERSDETAIL: members});
    }
    colConversation.doc(conversationData[kDbCONVERSATIONID]).update(data);
  }

  static setLastTimeTimestamp(conversationData, [timestamp]) async {
    if (conversationData == null) return;
    List members = conversationData[kDbMEMBERSDETAIL];
    members.firstWhere((e) => e[kDbUSERNAME] == loggedUsername)[kDbTIMESTAMP] =
        timestamp ??
            DateTime.now()
                .add(const Duration(seconds: 1))
                .millisecondsSinceEpoch;
    await colConversation
        .doc(conversationData[kDbCONVERSATIONID])
        .update({kDbMEMBERSDETAIL: members});
  }

  static removeConversation(id) async {
    colConversation.doc(id).delete();
  }

  static getWhen(DateTime date) {
    DateTime now = DateTime.now();
    String when;
    if (date.isToday()) {
      when = 'today'.tr;
    } else if (date.day == now.subtract(const Duration(days: 1)).day) {
      when = 'yesterday'.tr;
    } else {
      when = DateFormat.MMMd().format(date);
    }
    return when;
  }
}
